"""Base models for the dfhir app."""

from django.contrib.postgres.fields import ArrayField
from django.core.validators import RegexValidator
from django.db import models

from . import choices

PHONE_REGEX = RegexValidator(
    regex=r"^\+?1?\d{9,15}$",
    message="Phone number must be entered in the format: '+999999999'. Up to 15 digits allowed.",
)


class TimeStampedModel(models.Model):
    """An abstract base class model that provides self-updating 'created' and 'modified' fields."""

    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    @property
    def resource_type(self):
        """Return the resource type."""
        return self.__class__.__name__

    class Meta:
        """Meta options."""

        abstract = True


class BaseReference(TimeStampedModel):
    """Base reference class."""

    reference = models.CharField(max_length=255, null=True)
    type = models.CharField(max_length=255, null=True)
    identifier = models.ForeignKey("Identifier", on_delete=models.DO_NOTHING, null=True)
    display = models.CharField(max_length=255, null=True)

    class Meta:
        """Meta options."""

        abstract = True


class Reference(BaseReference):
    """Reference model."""


class Period(TimeStampedModel):
    """Period model."""

    start = models.DateTimeField(null=True)
    end = models.DateTimeField(null=True)


class ServiceType(TimeStampedModel):
    """ServiceType model."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)

    class Meta:
        """Meta options."""

        app_label = "healthcareservices"


class ContactPoint(TimeStampedModel):
    """ContactPoint model."""

    system = models.CharField(
        max_length=10,
        choices=choices.ContactPointSystemChoices.choices,
        blank=True,
        null=True,
        help_text="Telecommunications form for contact point.",
    )
    value = models.CharField(max_length=255, blank=True, null=True)
    use = models.CharField(
        max_length=10,
        choices=choices.ContactPointUseChoices.choices,
        blank=True,
        null=True,
    )
    rank = models.PositiveIntegerField(blank=True, null=True)
    period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="contact_point_period",
        null=True,
    )


class Telecom(TimeStampedModel):
    """Telecom model."""

    value = models.CharField(validators=[PHONE_REGEX], max_length=17, blank=True)
    use = models.CharField(
        max_length=10, choices=choices.TelecomUseChoices.choices, default="mobile"
    )

    class Meta:
        """Meta options."""

        abstract = True


class Address(TimeStampedModel):
    """Address model."""

    use = models.CharField(
        max_length=10, choices=choices.AddressUseChoices.choices, null=True
    )
    type = models.CharField(
        max_length=10, choices=choices.AddressTypeChoices.choices, null=True
    )
    text = models.CharField(max_length=255, null=True, blank=True)
    line = ArrayField(
        models.CharField(max_length=255, null=True, blank=True), null=True
    )
    city = models.CharField(max_length=255, null=True, blank=True)
    district = models.CharField(max_length=255, null=True, blank=True)
    state = models.CharField(max_length=255, null=True, blank=True)
    postal_code = models.CharField(max_length=255, null=True, blank=True)
    country = models.CharField(max_length=255, null=True, blank=True)
    period = models.ForeignKey(
        Period, on_delete=models.DO_NOTHING, related_name="address_period", null=True
    )


class ContactDetail(TimeStampedModel):
    """ContactDetail model."""

    name = models.CharField(max_length=255, null=True, blank=True)
    telecom = models.ManyToManyField(
        ContactPoint, related_name="contact_detail_telecom", blank=True
    )


class AvailableTime(TimeStampedModel):
    """AvailableTime model."""

    days_of_week = ArrayField(
        models.CharField(max_length=255, choices=choices.DaysOfWeekChoices.choices),
        null=True,
    )
    all_day = models.BooleanField(default=False)
    available_start_time = models.TimeField(null=True)
    available_end_time = models.TimeField(null=True)


class NotAvailableTime(TimeStampedModel):
    """NotAvailableTime model."""

    description = models.CharField(max_length=255, null=True, blank=True)
    during = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="not_available_time_during",
        null=True,
    )


class Availability(TimeStampedModel):
    """Availability model."""

    period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="availability_period",
        null=True,
    )
    available_time = models.ManyToManyField(
        AvailableTime, related_name="availability_available_time", blank=True
    )
    not_available_time = models.ManyToManyField(
        NotAvailableTime, related_name="availability_not_available_time", blank=True
    )


class HumanName(TimeStampedModel):
    """HumanName model."""

    use = models.CharField(
        max_length=255, choices=choices.HumanNameUseChoices.choices, null=True
    )
    text = models.TextField(null=True)
    family = models.CharField(max_length=255, null=True)
    given = ArrayField(models.CharField(max_length=255), null=True)
    prefix = ArrayField(models.CharField(max_length=255), null=True)
    suffix = ArrayField(models.CharField(max_length=255), null=True)
    period = models.ForeignKey(
        Period, on_delete=models.DO_NOTHING, related_name="human_name_period", null=True
    )


class ExtendedContactDetail(TimeStampedModel):
    """ExtendedContactDetail model."""

    system = models.CharField(
        max_length=10,
        choices=choices.TelecomSystemChoices.choices,
        blank=True,
        null=True,
        help_text="Telecommunications form for contact point.",
    )
    purpose = models.ForeignKey(
        "CodeableConcept",
        on_delete=models.DO_NOTHING,
        related_name="extended_contact_detail_purpose",
        null=True,
    )
    name = models.ManyToManyField(
        HumanName, related_name="extended_contact_detail_name", blank=True
    )
    address = models.ForeignKey(
        Address,
        on_delete=models.DO_NOTHING,
        related_name="extended_contact_detail_address",
        null=True,
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="extended_contact_detail_period",
        null=True,
    )
    telecom = models.ManyToManyField(
        ContactPoint, related_name="extended_contact_detail_telecom", blank=True
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="extended_contact_detail_organization",
        null=True,
    )


class Participant(TimeStampedModel):
    """Participants involved in the appointment."""

    type = models.CharField(
        max_length=255, choices=choices.ParticipantType.choices, null=True
    )
    status = models.CharField(
        max_length=255, choices=choices.ParticipantStatus.choices, null=True
    )

    class Meta:
        """Meta options."""

        abstract = True


class Attachment(TimeStampedModel):
    """diagnostic report attachment model."""

    title = models.CharField(max_length=255, blank=True, null=True)
    content_type = models.CharField(max_length=255, blank=True, null=True)
    language = models.CharField(max_length=255, blank=True, null=True)
    data = models.TextField(blank=True, null=True)
    url = models.URLField(blank=True, null=True)
    size = models.BigIntegerField(blank=True, null=True)
    hash = models.BinaryField(blank=True, null=True)
    creation = models.DateTimeField(blank=True, null=True)
    height = models.PositiveIntegerField(blank=True, null=True)
    width = models.PositiveIntegerField(blank=True, null=True)
    frames = models.PositiveIntegerField(blank=True, null=True)
    duration = models.DecimalField(
        max_digits=1000, decimal_places=2, blank=True, null=True
    )
    pages = models.PositiveIntegerField(blank=True, null=True)


class Coding(TimeStampedModel):
    """Coding model."""

    system = models.CharField(max_length=255, null=True)
    version = models.CharField(max_length=255, null=True)
    code = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)
    user_selected = models.BooleanField(default=False)


class CodeableConcept(TimeStampedModel):
    """CodeableConcept model."""

    text = models.CharField(max_length=255, null=True)
    coding = models.ManyToManyField(Coding, related_name="codeable_concept")


class OrganizationReference(BaseReference):
    """Organization Reference model."""

    identifier = models.ForeignKey(
        "Identifier",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="organization_reference_identifier",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="organization_reference",
        null=True,
    )


class Identifier(TimeStampedModel):
    """Identifier model."""

    use = models.CharField(max_length=255, null=True)
    type = models.ForeignKey(CodeableConcept, on_delete=models.DO_NOTHING, null=True)
    system = models.CharField(max_length=255, null=True)
    value = models.CharField(max_length=255, null=True)
    period = models.ForeignKey(
        Period, on_delete=models.DO_NOTHING, related_name="identifier", null=True
    )
    assigner = models.ForeignKey(
        "organizationReference",
        on_delete=models.DO_NOTHING,
        related_name="identifier_for_organization",
        null=True,
    )


class Qualification(TimeStampedModel):
    """Qualification model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="qualification_identifier", blank=True
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="qualification_code",
        null=True,
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="qualification_period",
        null=True,
    )
    issuer = models.ForeignKey(
        "OrganizationReference",
        on_delete=models.DO_NOTHING,
        related_name="qualification_issuer",
        null=True,
    )


class ConnectionType(TimeStampedModel):
    """ConnectionType model."""

    code = models.CharField(max_length=255, null=True)
    system = models.CharField(max_length=255, null=True)
    display = models.CharField(max_length=255, null=True)
    definition = models.TextField(null=True)


class Payload(TimeStampedModel):
    """Payload model."""

    type = models.ManyToManyField(
        CodeableConcept, related_name="payload_type", blank=True
    )
    # mimetype = models.CharField(max_length=255, null=True) #TODO: needs a revisit
    profile_uri = ArrayField(models.URLField(max_length=255), null=True)
    profile_canonical = ArrayField(models.URLField(max_length=255), null=True)


class VirtualServiceDetailAddress(TimeStampedModel):
    """Connection details of a virtual service (e.g. conference call)."""

    address_url = models.URLField(max_length=255, null=True)
    address_string = models.TextField(null=True)
    address_contact_point = models.ForeignKey(
        ContactPoint,
        on_delete=models.DO_NOTHING,
        related_name="virtual_service_detail_address_contact_point",
        null=True,
    )
    address_extended_contact_detail = models.ForeignKey(
        ExtendedContactDetail,
        on_delete=models.DO_NOTHING,
        related_name="virtual_service_detail_address_extended_contact_detail",
        null=True,
    )


class VirtualServiceDetails(TimeStampedModel):
    """Connection details of a virtual service (e.g. conference call)."""

    channel_type = models.ForeignKey(
        Coding,
        on_delete=models.DO_NOTHING,
        related_name="virtual_service_details_channel_type",
        null=True,
    )
    address = models.ForeignKey(
        VirtualServiceDetailAddress,
        on_delete=models.DO_NOTHING,
        related_name="virtual_service_details_address",
        null=True,
    )
    additional_info = ArrayField(models.URLField(max_length=255), null=True)
    max_participants = models.PositiveIntegerField(null=True)
    session_key = models.CharField(max_length=255, null=True)


class Quantity(TimeStampedModel):
    """Quantity model."""

    value = models.DecimalField(max_digits=1000, decimal_places=2, null=True)
    comparator = models.CharField(
        max_length=255, choices=choices.QuantityComparatorChoices.choices, null=True
    )
    unit = models.CharField(max_length=255, null=True)
    system = models.CharField(max_length=255, null=True)
    code = models.CharField(max_length=255, null=True)


class Range(TimeStampedModel):
    """Range model."""

    low = models.ForeignKey(
        Quantity, on_delete=models.DO_NOTHING, related_name="range_low", null=True
    )
    high = models.ForeignKey(
        Quantity, on_delete=models.DO_NOTHING, related_name="range_high", null=True
    )


class Annotation(TimeStampedModel):
    """Annotation model."""

    author_reference = models.ForeignKey(
        Reference,
        on_delete=models.DO_NOTHING,
        related_name="annotation_author_reference",
        null=True,
    )
    author_string = models.CharField(max_length=255, null=True)
    time = models.DateTimeField(null=True)
    text = models.TextField(null=True)


class Communication(TimeStampedModel):
    """communication model."""

    language = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="communication_language",
        null=True,
    )
    preferred = models.BooleanField(default=False)


class Ratio(TimeStampedModel):
    """ratio model."""

    numerator = models.ForeignKey(
        Quantity, on_delete=models.DO_NOTHING, null=True, related_name="ratio_numerator"
    )
    denominator = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="ratio_denominator",
    )


class CodeableReference(TimeStampedModel):
    """codeable reference."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="codeable_reference_concept",
    )
    reference = models.ForeignKey(
        Reference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="codeable_reference_reference",
    )


class Repeat(TimeStampedModel):
    """repeat model."""

    bounds_duration = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="repeat_bounds_duration",
        null=True,
    )
    bounds_range = models.ForeignKey(
        Range,
        on_delete=models.DO_NOTHING,
        related_name="repeat_bounds_range",
        null=True,
    )
    bounds_period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        related_name="repeat_bounds_period",
        null=True,
    )
    count = models.PositiveIntegerField(null=True)
    count_max = models.PositiveIntegerField(null=True)
    duration = models.DecimalField(null=True, max_digits=1000, decimal_places=2)
    duration_max = models.DecimalField(null=True, max_digits=1000, decimal_places=2)
    duration_unit = models.CharField(
        max_length=255, null=True, choices=choices.RepeatDurationUnits.choices
    )
    frequency = models.PositiveIntegerField(null=True)
    frequency_max = models.PositiveIntegerField(null=True)
    period = models.DecimalField(null=True, max_digits=1000, decimal_places=2)
    period_max = models.DecimalField(null=True, max_digits=1000, decimal_places=2)
    period_unit = models.CharField(
        max_length=255, null=True, choices=choices.RepeatDurationUnits.choices
    )
    day_of_week = models.ManyToManyField(
        Coding,
        blank=True,
        related_name="repeat_day_of_week",
    )
    time_of_day = models.TimeField(null=True)
    when = models.ManyToManyField(Coding, blank=True, related_name="repeat_when")


class Timing(TimeStampedModel):
    """timing model."""

    event = models.DateTimeField(null=True)
    repeat = models.ForeignKey(
        Repeat, on_delete=models.DO_NOTHING, null=True, related_name="timing_repeat"
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="timing_code",
    )


class RelatedArtifact(TimeStampedModel):
    """RelatedArtifact model."""

    type = models.CharField(
        max_length=255, choices=choices.RelatedArtifactTypeChoices.choices, null=True
    )
    classifier = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="related_artifact_classifier",
        null=True,
    )
    label = models.CharField(max_length=255, null=True)

    display = models.TextField(null=True)
    citation = models.TextField(null=True)
    document = models.ForeignKey(
        Attachment,
        on_delete=models.DO_NOTHING,
        related_name="related_artifact_document",
        null=True,
    )
    # resource = models.ForeignKey(  # TODO: this should be canonical
    #     Reference,
    #     on_delete=models.DO_NOTHING,
    #     related_name="related_artifact_resource",
    #     null=True,
    # )
    resource_reference = models.ForeignKey(
        Reference,
        on_delete=models.DO_NOTHING,
        related_name="related_artifact_resource",
        null=True,
    )
    publication_status = models.CharField(
        max_length=255, null=True, choices=choices.PublicationStatusChoices.choices
    )
    publication_date = models.DateTimeField(null=True)


class UsageContext(TimeStampedModel):
    """UsageContext model."""

    code = models.ForeignKey(
        Coding,
        on_delete=models.DO_NOTHING,
        related_name="usage_context_code",
        null=True,
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="usage_context_value",
        null=True,
    )
    value_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="usage_context_value_quantity",
        null=True,
    )
    value_range = models.ForeignKey(
        Range,
        on_delete=models.DO_NOTHING,
        related_name="usage_context_value_range",
        null=True,
    )
    # value_reference = models.ForeignKey(
    #     Reference,
    #     on_delete=models.DO_NOTHING,
    #     related_name="usage_context_value_reference",
    #     null=True,
    # )


class ProductShelfLife(TimeStampedModel):
    """ProductShelfLife model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="product_shelf_life_type",
        null=True,
    )
    period_duration = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="product_shelf_life_period_duration",
        null=True,
    )
    period_string = models.CharField(max_length=255, null=True)
    special_precautions_for_storage = models.ManyToManyField(
        CodeableConcept,
        related_name="product_shelf_life_special_precautions_for_storage",
        blank=True,
    )


class Expression(TimeStampedModel):
    """Expression model."""

    description = models.TextField(null=True)
    name = models.CharField(max_length=255, null=True)
    language = models.CharField(max_length=255, null=True)
    expression = models.TextField(null=True)
    reference = models.CharField(max_length=255, null=True)


class RelativeTime(TimeStampedModel):
    """Relative time model."""

    context_reference = models.ForeignKey(
        Reference,
        on_delete=models.DO_NOTHING,
        related_name="relative_time_context_reference",
        null=True,
    )
    # TODO: fix this
    # context_definition = models.ForeignKey(CodeableConcept, on_delete=models.DO_NOTHING, related_name="relative_time_context_definition", null=True)
    context_path = models.CharField(max_length=255, null=True)
    context_code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="relative_time_context_code",
        null=True,
    )
    offset_duration = models.ForeignKey(
        Quantity,
        on_delete=models.DO_NOTHING,
        related_name="relative_time_offset_duration",
        null=True,
    )
    offset_range = models.ForeignKey(
        Range,
        on_delete=models.DO_NOTHING,
        related_name="relative_time_offset_range",
        null=True,
    )
    text = models.TextField(null=True)


class Age(Quantity):
    """Age model."""

    pass


class Duration(Quantity):
    """Duration model."""

    pass


class SimpleQuantity(Quantity):
    """SimpleQuantity model."""

    pass


class Money(TimeStampedModel):
    """Money model."""

    value = models.DecimalField(max_digits=1000, decimal_places=2, null=True)
    currency = models.CharField(max_length=255, null=True)


class MonetaryComponent(TimeStampedModel):
    """MonetaryComponent model."""

    type = models.CharField(
        max_length=255, null=True, choices=choices.MonetaryComponentChoices.choices
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="monetory_component_code",
        null=True,
    )
    factor = models.DecimalField(max_digits=1000, decimal_places=2, null=True)
    amount = models.ForeignKey(
        Money,
        on_delete=models.DO_NOTHING,
        related_name="monetory_component_amount",
        null=True,
    )


class SignatureWhoReference(BaseReference):
    """Signature who reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="signature_who_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="signature_who_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="signature_who_reference_practitioner_role",
        null=True,
    )
    related_person = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="signature_who_reference_related_person",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="signature_who_reference_patient",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        related_name="signature_who_reference_device",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="signature_who_reference_organization",
        null=True,
    )


class SignatureOnBehalfOfReference(BaseReference):
    """Signature on behalf of reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="signature_on_behalf_of_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        related_name="signature_on_behalf_of_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        related_name="signature_on_behalf_of_reference_practitioner_role",
        null=True,
    )
    related_person = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="signature_on_behalf_of_reference_related_person",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="signature_on_behalf_of_reference_patient",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        related_name="signature_on_behalf_of_reference_device",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        related_name="signature_on_behalf_of_reference_organization",
        null=True,
    )


class Signature(TimeStampedModel):
    """Signature model."""

    type = models.ManyToManyField(Coding, related_name="signature_type", blank=True)
    when = models.DateTimeField(null=True)
    who = models.ForeignKey(
        SignatureWhoReference,
        on_delete=models.DO_NOTHING,
        related_name="signature_who",
        null=True,
    )
    on_behalf_of = models.ForeignKey(
        SignatureOnBehalfOfReference,
        on_delete=models.DO_NOTHING,
        related_name="signature_on_behalf_of",
        null=True,
    )
    target_format = models.CharField(max_length=255, null=True)
    sig_format = models.CharField(max_length=255, null=True)
    data = models.TextField(null=True)


class TriggerDefinition(TimeStampedModel):
    """TriggerDefinition model."""

    type = models.CharField(
        max_length=255, choices=choices.TriggerDefinitionTypeChoices.choices, null=True
    )
    name = models.CharField(max_length=255, null=True)
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="trigger_definition_code",
        null=True,
    )
    # subscription_topic = models.ForeignKey(
    #     CanonicalSubscriptionTopic,
    #     on_delete=models.DO_NOTHING,
    #     related_name="trigger_definition_subscription_topic",
    #     null=True,
    # )
    timing_timing = models.ForeignKey(
        Timing,
        on_delete=models.DO_NOTHING,
        related_name="trigger_definition_timing_timing",
        null=True,
    )
    timing_reference = models.ForeignKey(
        "schedules.ScheduleReference",
        on_delete=models.DO_NOTHING,
        related_name="trigger_definition_timing_reference",
        null=True,
    )
    timing_date = models.DateField(null=True)
    timing_date_time = models.DateTimeField(null=True)
    # data = models.ForeignKey(
    #     "DataRequirement",
    #     on_delete=models.DO_NOTHING,
    #     related_name="trigger_definition_data",
    #     null=True,
    # )
    condition = models.ForeignKey(
        Expression,
        on_delete=models.DO_NOTHING,
        related_name="trigger_definition_condition",
        null=True,
    )


# class DataRequirement(TimeStampedModel):
#     """DataRequirement model."""
#
#     type = models.CharField(max_length=255, null=True)
#     # profile = models.ManyToManyField(
#     #     CanonicalStructureDefinition, related_name="data_requirement_profile", blank=True
#     # )
#     subject_codeable_concept = models.ForeignKey(
#         CodeableConcept,
#         on_delete=models.DO_NOTHING,
#         related_name="data_requirement_subject_codeable_concept",
#         null=True,
#     )
#     subject_reference = models.ForeignKey(
#         "groups.GroupReference",
#         on_delete=models.DO_NOTHING,
#         related_name="data_requirement_subject_reference",
#         null=True,
#     )
#     must_support = ArrayField(models.CharField(max_length=255), null=True)



"""body structures serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    QuantitySerializer,
)
from dfhir.bodystructures.models import (
    BodyLandmarkOrientation,
    BodyStructure,
    BodyStructureCodeableReference,
    BodyStructureReference,
    DistanceFromLandmark,
    IncludedStructure,
)
from dfhir.devices.serializers import DeviceReferenceSerializer
from dfhir.imagingselections.serializers import ImagingSelectionReferenceSerializer
from dfhir.patients.serializers import PatientReferenceSerializer


class DistanceFromLandmarkSerializer(WritableNestedModelSerializer):
    """Distance from landmark serializer."""

    device = DeviceReferenceSerializer(many=True, required=False)
    value = QuantitySerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = DistanceFromLandmark
        exclude = ["created_at", "updated_at"]


class BodyLandmarkOrientationSerializer(WritableNestedModelSerializer):
    """Body landmark orientation serializer."""

    landmark_description = CodeableConceptSerializer(many=True, required=False)
    clock_face_position = CodeableConceptSerializer(many=True, required=False)
    distance_from_landmark = DistanceFromLandmarkSerializer(many=True, required=False)
    surface_orientation = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = BodyLandmarkOrientation
        exclude = ["created_at", "updated_at"]


class IncludedStructureSerializer(WritableNestedModelSerializer):
    """Included structure serializer."""

    structure = CodeableConceptSerializer(many=False, required=False)
    laterality = CodeableConceptSerializer(many=False, required=False)
    body_landmark_orientation = BodyLandmarkOrientationSerializer(
        many=True, required=False
    )
    spatial_reference = ImagingSelectionReferenceSerializer(many=True, required=False)
    qualifier = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = IncludedStructure
        exclude = ["created_at", "updated_at"]


class BodyStructureReferenceSerializer(BaseReferenceModelSerializer):
    """Body structure reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = BodyStructureReference
        exclude = ["created_at", "updated_at"]


class BodyStructureSerializer(BaseWritableNestedModelSerializer):
    """Body structure serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    morphology = CodeableConceptSerializer(many=False, required=False)
    included_structure = IncludedStructureSerializer(many=True, required=False)
    excluded_structure = IncludedStructureSerializer(many=True, required=False)
    image = AttachmentSerializer(many=True, required=False)
    patient = PatientReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = BodyStructure
        exclude = ["created_at", "updated_at"]


class BodyStructureCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Body structure codeable reference serializer."""

    reference = BodyStructureReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = BodyStructureCodeableReference
        exclude = ["created_at", "updated_at"]



"""Episode of care serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.accounts.serializers import AccountReferenceSerializer
from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
)
from dfhir.careteams.serializers import CareTeamReferenceSerializer
from dfhir.conditions.serializers import ConditionCodeableReferenceSerializer
from dfhir.patients.serializers import PatientGroupReferenceSerializer
from dfhir.practitioners.serializers import (
    PractitionerPractitionerRoleReferenceSerializer,
)
from dfhir.servicerequests.serializers import ServiceRequestReferenceSerializer

from .models import (
    EpisodeOfCare,
    EpisodeOfCareDiagnosis,
    EpisodeOfCareReason,
    EpisodeOfCareReasonValueCodeableReference,
    EpisodeOfCareReasonValueReference,
    EpisodeOfCareReference,
    EpisodeOfCareStatusHistory,
)


class EpisodeOfCareReferenceSerializer(BaseReferenceModelSerializer):
    """Episode of care reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = EpisodeOfCareReference
        exclude = ["created_at", "updated_at"]


class EpisodeOfCareStatusHistorySerializer(WritableNestedModelSerializer):
    """Episode of care status history serializer."""

    period = PeriodSerializer(required=False)

    class Meta:
        """Meta class."""

        model = EpisodeOfCareStatusHistory
        exclude = ["created_at", "updated_at"]


class EpisodeOfCareReasonValueReferenceSerializer(BaseReferenceModelSerializer):
    """Episode of care reason value reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = EpisodeOfCareReasonValueReference
        exclude = ["created_at", "updated_at"]


class EpisodeOfCareReasonValueCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """Episode of care reason value codeable reference serializer."""

    reference = EpisodeOfCareReasonValueReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = EpisodeOfCareReasonValueCodeableReference
        exclude = ["created_at", "updated_at"]


class EpisodeOfCareReasonSerializer(WritableNestedModelSerializer):
    """Episode of care reason serializer."""

    use = CodeableConceptSerializer(many=False, required=False)
    value = EpisodeOfCareReasonValueCodeableReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """Meta class."""

        model = EpisodeOfCareReason
        exclude = ["created_at", "updated_at"]


class EpisodeOfCareDiagnosisSerializer(WritableNestedModelSerializer):
    """Episode of care diagnosis serializer."""

    condition = ConditionCodeableReferenceSerializer(many=False, required=False)
    use = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = EpisodeOfCareDiagnosis
        exclude = ["created_at", "updated_at"]


class EpisodeOfCareSerializer(BaseWritableNestedModelSerializer):
    """Episode of care serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    status_history = EpisodeOfCareStatusHistorySerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=True, required=False)
    diagnosis = EpisodeOfCareDiagnosisSerializer(many=True, required=False)
    subject = PatientGroupReferenceSerializer(many=False, required=False)
    managing_organization = OrganizationReferenceSerializer(many=False, required=False)
    period = PeriodSerializer(many=False, required=False)
    care_manager = PractitionerPractitionerRoleReferenceSerializer(
        many=False, required=False
    )
    care_team = CareTeamReferenceSerializer(many=True, required=False)
    referral_request = ServiceRequestReferenceSerializer(many=True, required=False)
    account = AccountReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = EpisodeOfCare
        exclude = ["created_at", "updated_at"]



"""DeviceDispenses choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class DeviceDispenseStatus(models.TextChoices):
    """DeviceDispenseStatus choices."""

    IN_PROGRESS = "in-progress", _("In Progress")
    COMPLETED = "completed", _("Completed")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    STOPPED = "stopped", _("Stopped")
    ON_HOLD = "on-hold", _("On Hold")
    CANCELLED = "cancelled", _("Cancelled")
    PREPARATION = "preparation", _("Preparation")
    DECLINED = "declined", _("Declined")



"""molecular definitions app."""

from django.apps import AppConfig


class MoleculardefinitionsConfig(AppConfig):
    """molecular definitions app config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.moleculardefinitions"
    verbose_name = "Molecular Definitions"



"""RelatedPersons models."""

from django.db import models

from dfhir.base import choices as base_choices
from dfhir.base.models import (
    Address,
    Attachment,
    BaseReference,
    CodeableConcept,
    Communication,
    ContactPoint,
    HumanName,
    Identifier,
    Period,
    TimeStampedModel,
)


class RelatedPerson(TimeStampedModel):
    """RelatedPerson model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="relatedpersons_identifier", blank=True
    )
    active = models.BooleanField(default=True)
    patient = models.ForeignKey(
        "patients.PatientReference",
        related_name="relatedpersons_patient",
        on_delete=models.CASCADE,
    )
    relationship = models.ManyToManyField(
        CodeableConcept, related_name="relatedpersons_relationship", blank=True
    )
    role = models.ManyToManyField(
        CodeableConcept, related_name="relatedpersons_role", blank=True
    )
    name = models.ManyToManyField(
        HumanName, related_name="relatedpersons_name", blank=True
    )
    telecom = models.ManyToManyField(
        ContactPoint, related_name="relatedpersons_telecom", blank=True
    )
    gender = models.CharField(
        max_length=20, choices=base_choices.GenderChoices.choices, null=True, blank=True
    )
    birth_date = models.DateField(null=True)
    address = models.ManyToManyField(
        Address, related_name="relatedpersons_address", blank=True
    )
    photo = models.ManyToManyField(
        Attachment, related_name="relatedpersons_photo", blank=True
    )
    period = models.ForeignKey(
        Period,
        related_name="relatedpersons_period",
        on_delete=models.CASCADE,
        null=True,
    )
    communication = models.ManyToManyField(
        Communication, related_name="relatedpersons_communication", blank=True
    )


class RelatedPersonReference(BaseReference):
    """RelatedPerson Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="related_person_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    related_person = models.ForeignKey(
        RelatedPerson,
        related_name="related_person_reference_related_person",
        on_delete=models.CASCADE,
        null=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('allergyintolerances', '0004_initial'),
        ('base', '0004_initial'),
        ('conditions', '0002_initial'),
        ('devices', '0001_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='condition',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='condition',
            name='verification_status',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_verification_status', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='conditionallergyintolerancereference',
            name='allergy_intolerance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_allergy_intolerance_reference_allergy_intolerance', to='allergyintolerances.allergyintolerance'),
        ),
        migrations.AddField(
            model_name='conditionallergyintolerancereference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_allergy_intolerance_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='conditionallergyintolerancereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_allergy_intolerance_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='conditionasserterreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_asserter_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='conditionasserterreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_asserter_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='conditionasserterreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_asserter_reference_patient', to='patients.patient'),
        ),
    ]



"""task serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    CodeableConceptSerializer,
    CodeableReferenceSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    ReferenceSerializer,
)
from dfhir.coverages.serializers import CoverageClaimResponseReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.provenances.serializers import ProvenanceReferenceSerializer
from dfhir.tasks.models import (
    Task,
    TaskInput,
    TaskOutput,
    TaskOwnerReference,
    TaskPerformer,
    TaskPerformerActorReference,
    TaskReference,
    TaskRequestedPerformerCodeableReference,
    TaskRequestedPerformerReference,
    TaskRequesterReference,
    TaskRestriction,
    TaskRestrictionRecipientReference,
)


class TaskReferenceSerializer(BaseReferenceModelSerializer):
    """Task reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskReference
        exclude = ["created_at", "updated_at"]


class TaskRequesterReferenceSerializer(BaseReferenceModelSerializer):
    """Task requester reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskRequesterReference
        exclude = ["created_at", "updated_at"]


class TaskRequestedPerformerReferenceSerializer(BaseReferenceModelSerializer):
    """Task requested performer reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskRequestedPerformerReference
        exclude = ["created_at", "updated_at"]


class TaskRequestedPerformerCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Task requested performer codeable reference serializer."""

    concept = CodeableConceptSerializer(required=False)
    reference = TaskRequestedPerformerReferenceSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskRequestedPerformerCodeableReference
        exclude = ["created_at", "updated_at"]


class TaskOwnerReferenceSerializer(BaseReferenceModelSerializer):
    """Task owner reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskOwnerReference
        exclude = ["created_at", "updated_at"]


class TaskPerformerActorReferenceSerializer(BaseReferenceModelSerializer):
    """Task performer actor reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskPerformerActorReference
        exclude = ["created_at", "updated_at"]


class TaskRestrictionRecipientReferenceSerializer(BaseReferenceModelSerializer):
    """Task restriction recipient reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskRestrictionRecipientReference
        exclude = ["created_at", "updated_at"]


class TaskPerformerSerializer(WritableNestedModelSerializer):
    """Task performer serializer."""

    function = CodeableConceptSerializer(required=False)
    actor = TaskPerformerActorReferenceSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskPerformer
        exclude = ["created_at", "updated_at"]


class TaskRestrictionSerializer(WritableNestedModelSerializer):
    """Task restriction serializer."""

    period = PeriodSerializer(required=False)
    recipient = TaskRestrictionRecipientReferenceSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskRestriction
        exclude = ["created_at", "updated_at"]


class TaskInputSerializer(WritableNestedModelSerializer):
    """Task input serializer."""

    type = CodeableConceptSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskInput
        exclude = ["created_at", "updated_at"]


class TaskOutputSerializer(WritableNestedModelSerializer):
    """Task output serializer."""

    type = CodeableConceptSerializer(required=False)

    class Meta:
        """Meta class."""

        model = TaskOutput
        exclude = ["created_at", "updated_at"]


class TaskSerializer(WritableNestedModelSerializer):
    """Task serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    # instantiates_canonical = TaskReferenceSerializer(required=False,many=True)
    based_on = ReferenceSerializer(required=False, many=True)
    group_identifier = IdentifierSerializer(required=False)
    part_of = TaskReferenceSerializer(required=False, many=True)
    status_reason = CodeableConceptSerializer(required=False)
    business_status = CodeableConceptSerializer(required=False)
    code = CodeableConceptSerializer(required=False)
    focus = ReferenceSerializer(required=False, many=False)
    for_value = ReferenceSerializer(required=False, many=False)
    encounter = EncounterReferenceSerializer(required=False)
    requested_period = PeriodSerializer(required=False)
    execution_period = PeriodSerializer(required=False)
    requester = TaskRequesterReferenceSerializer(required=False)
    requested_performer = TaskRequestedPerformerCodeableReferenceSerializer(
        required=False, many=True
    )
    owner = TaskOwnerReferenceSerializer(required=False)
    performer = TaskPerformerSerializer(required=False, many=True)
    location = LocationReferenceSerializer(required=False)
    reason = CodeableReferenceSerializer(required=False, many=True)
    insurance = CoverageClaimResponseReferenceSerializer(required=False, many=True)
    note = AnnotationSerializer(required=False, many=True)
    relevant_history = ProvenanceReferenceSerializer(required=False, many=True)
    restriction = TaskRestrictionSerializer(required=False)
    input = TaskInputSerializer(required=False, many=True)
    output = TaskOutputSerializer(required=False, many=True)

    class Meta:
        """Meta class."""

        model = Task
        exclude = ["created_at", "updated_at"]
        rename_fields = {
            "for": "for_value",
        }



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('conditions', '0003_initial'),
        ('devices', '0001_initial'),
        ('diagnosticreports', '0002_initial'),
        ('encounters', '0003_initial'),
        ('observations', '0002_initial'),
        ('patients', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='RiskAssessmentPerformerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_performer_references_device', to='devices.device')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_performer_references_identifier', to='base.identifier')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_performer_references_patient', to='patients.patient')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_performer_references_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_performer_references_practitioner_role', to='practitionerroles.practitionerrole')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_performer_references_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RiskAssessmentPrediction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('probability_decimal', models.DecimalField(decimal_places=10, max_digits=10, null=True)),
                ('relative_risk', models.DecimalField(decimal_places=10, max_digits=10, null=True)),
                ('rationale', models.TextField(null=True)),
                ('outcome', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_predictions_outcome', to='base.codeableconcept')),
                ('probability_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_predictions_probability_range', to='base.range')),
                ('qualitative_risk', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_predictions_qualitative_risk', to='base.codeableconcept')),
                ('when_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_predictions_when_period', to='base.period')),
                ('when_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_predictions_when_range', to='base.range')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RiskAssessmentReasonCodeableConcept',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_reason_codeable_concepts_code', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RiskAssessment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('registered', 'Registered'), ('specimen-in-progress', 'Specimen in Progress'), ('preliminary', 'Preliminary'), ('final', 'Final'), ('amended', 'Amended'), ('corrected', 'Corrected'), ('appended', 'Appended'), ('cancelled', 'Cancelled'), ('entered-in-error', 'Entered in Error'), ('unknown', 'Unknown'), ('cannot-be-obtained', 'Con not be Obtained')], max_length=200, null=True)),
                ('occurrence_date_time', models.DateTimeField(null=True)),
                ('mitigation', models.CharField(max_length=200, null=True)),
                ('based_on', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessments_based_on', to='base.reference')),
                ('basis', models.ManyToManyField(blank=True, related_name='risk_assessments_basis', to='base.reference')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessments_code', to='base.codeableconcept')),
                ('condition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessments_condition', to='conditions.conditionreference')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessments_encounter', to='encounters.encounterreference')),
                ('identifier', models.ManyToManyField(blank=True, related_name='risk_assessments_identifier', to='base.identifier')),
                ('method', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessments_method', to='base.codeableconcept')),
                ('note', models.ManyToManyField(blank=True, related_name='risk_assessments_note', to='base.annotation')),
                ('occurrence_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessments_occurrence_period', to='base.period')),
                ('parent', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessments_parent', to='base.reference')),
                ('subject', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessments_subject', to='patients.patientgroupreference')),
                ('performer', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessments_performer', to='riskassessments.riskassessmentperformerreference')),
                ('prediction', models.ManyToManyField(blank=True, related_name='risk_assessments_prediction', to='riskassessments.riskassessmentprediction')),
                ('reason', models.ManyToManyField(blank=True, related_name='risk_assessments_reason', to='riskassessments.riskassessmentreasoncodeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='RiskAssessmentReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('condition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_reason_references_condition', to='conditions.condition')),
                ('diagnostic_report', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_reason_references_diagnostic_report', to='diagnosticreports.diagnosticreport')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_reason_references_identifier', to='base.identifier')),
                ('observation', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_reason_references_observation', to='observations.observation')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='riskassessmentreasoncodeableconcept',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_reason_codeable_concepts_reference', to='riskassessments.riskassessmentreasonreference'),
        ),
    ]



"""medication dispense serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    QuantitySerializer,
    ReferenceSerializer,
)
from dfhir.careplans.serializers import CarePlanReferenceSerializer
from dfhir.detectedissues.serializers import DetectedIssueCodeableReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.medicationdispenses.models import (
    MedicationDispense,
    MedicationDispensePerformer,
    MedicationDispensePerformerActorReference,
    MedicationDispenseReceiverReference,
    MedicationDispenseSubstitution,
    MedicationDispenseSubstitutionResponsiblePartyReference,
)
from dfhir.medicationrequests.serializers import MedicationRequestReferenceSerializer
from dfhir.medications.serializers import MedicationCodeableReferenceSerializer
from dfhir.patients.serializers import PatientGroupReferenceSerializer
from dfhir.procedures.serializers import ProcedureReferenceSerializer
from dfhir.provenances.serializers import ProvenanceSerializer


class MedicationDispensePerformerActorReferenceSerializer(BaseReferenceModelSerializer):
    """medication dispense performer actor reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationDispensePerformerActorReference
        exclude = ["created_at", "updated_at"]


class MedicationDispensePerformerSerializer(WritableNestedModelSerializer):
    """medication dispense performer serializer."""

    function = CodeableConceptSerializer(many=False, required=False)
    actor = MedicationDispensePerformerActorReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """meta options."""

        model = MedicationDispensePerformer
        exclude = ["created_at", "updated_at"]


class MedicationDispenseReceiverReferenceSerializer(BaseReferenceModelSerializer):
    """medication dispense receiver reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationDispenseReceiverReference
        exclude = ["created_at", "updated_at"]


class MedicationDispenseSubstitutionResponsiblePartyReferenceSerializer(
    BaseReferenceModelSerializer
):
    """medication dispense substitution responsible party reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """meta options."""

        model = MedicationDispenseSubstitutionResponsiblePartyReference
        exclude = ["created_at", "updated_at"]


class MedicationDispenseSubstitutionSerializer(WritableNestedModelSerializer):
    """medication dispense substitution serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    reason = CodeableConceptSerializer(many=True, required=False)
    responsible_party = (
        MedicationDispenseSubstitutionResponsiblePartyReferenceSerializer(
            many=True, required=False
        )
    )

    class Meta:
        """meta options."""

        model = MedicationDispenseSubstitution
        exclude = ["created_at", "updated_at"]


class MedicationDispenseSerializer(BaseWritableNestedModelSerializer):
    """medication dispense serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    based_on = CarePlanReferenceSerializer(many=True, required=False)
    part_of = ProcedureReferenceSerializer(many=True, required=False)
    not_performed_reason = DetectedIssueCodeableReferenceSerializer(
        many=False, required=False
    )
    category = CodeableConceptSerializer(many=True, required=False)
    medication = MedicationCodeableReferenceSerializer(many=False, required=False)
    subject = PatientGroupReferenceSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    supporting_information = ReferenceSerializer(many=True, required=False)
    performer = MedicationDispensePerformerSerializer(many=True, required=False)
    location = LocationReferenceSerializer(many=False, required=False)
    authorizing_prescription = MedicationRequestReferenceSerializer(
        many=True, required=False
    )
    type = CodeableConceptSerializer(many=False, required=False)
    quantity = QuantitySerializer(many=False, required=False)
    days_supply = QuantitySerializer(many=False, required=False)
    destination = LocationReferenceSerializer(many=False, required=False)
    receiver = MedicationDispenseReceiverReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)
    substitution = MedicationDispenseSubstitutionSerializer(many=False, required=False)
    event_history = ProvenanceSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = MedicationDispense
        exclude = ["created_at", "updated_at"]



"""CareTeam serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    ContactPointSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    TimingSerializer,
)
from dfhir.conditions.serializers import ConditionCodeableReferenceSerializer
from dfhir.patients.serializers import PatientGroupReferenceSerializer

from .models import (
    CareTeam,
    CareTeamParticipant,
    CareTeamParticipantMemberReference,
    CareTeamParticipantOnBehalfOfReference,
    CareTeamReference,
)


class CareTeamParticipantMemberReferenceSerializer(BaseReferenceModelSerializer):
    """Care Team Participant Member Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = CareTeamParticipantMemberReference
        exclude = ["created_at", "updated_at"]


class CareTeamParticipantOnBehalfOfReferenceSerializer(BaseReferenceModelSerializer):
    """Care Team Participant On Behalf Of Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = CareTeamParticipantOnBehalfOfReference
        exclude = ["created_at", "updated_at"]


class CareTeamParticipantSerializer(WritableNestedModelSerializer):
    """Care Team Participant serializer."""

    role = CodeableConceptSerializer(required=False)
    member = CareTeamParticipantMemberReferenceSerializer(required=False)
    on_behalf_of = CareTeamParticipantOnBehalfOfReferenceSerializer(required=False)
    effective_period = PeriodSerializer(required=False)
    effective_timing = TimingSerializer(required=False)

    class Meta:
        """Meta class."""

        model = CareTeamParticipant
        exclude = ["created_at", "updated_at"]


class CareTeamSerializer(BaseWritableNestedModelSerializer):
    """Care Team serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    subject = PatientGroupReferenceSerializer(required=False)
    period = PeriodSerializer(required=False)
    participant = CareTeamParticipantSerializer(many=True, required=False)
    reason = ConditionCodeableReferenceSerializer(many=True, required=False)
    managing_organization = OrganizationReferenceSerializer(many=True, required=False)
    telecom = ContactPointSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = CareTeam
        exclude = ["created_at", "updated_at"]


class CareTeamReferenceSerializer(BaseReferenceModelSerializer):
    """Care Team Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = CareTeamReference
        exclude = ["created_at", "updated_at"]



"""Devicerequests serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    QuantitySerializer,
    RangeSerializer,
    ReferenceSerializer,
    TimingSerializer,
)

# from dfhir.devices.serializers import DeviceDeviceDefinitionCodeableReferenceSerializer
# from dfhir.coverages.serializers import CoverageClaimReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.provenances.serializers import ProvenanceReferenceSerializer

from .models import (
    DeviceRequest,
    DeviceRequestParameter,
    DeviceRequestPerformerCodeableReference,
    DeviceRequestPerformerReference,
    DeviceRequestReasonCodeableReference,
    DeviceRequestReasonReference,
    DeviceRequestReference,
    DeviceRequestRequesterCodeableReference,
    DeviceRequestRequesterReference,
    DeviceRequestSubjectReference,
)


class DeviceRequestReferenceSerializer(BaseReferenceModelSerializer):
    """Device Request Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = DeviceRequestReference
        exclude = ["created_at", "updated_at"]


class DeviceRequestParameterSerializer(WritableNestedModelSerializer):
    """Device Request Parameter Serializer."""

    code = CodeableConceptSerializer(many=False, required=False)
    value_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    value_quantity = QuantitySerializer(many=False, required=False)
    value_range = RangeSerializer(many=False, required=False)

    class Meta:
        """Meta."""

        model = DeviceRequestParameter
        exclude = ["created_at", "updated_at"]


class DeviceRequestPerformerReferenceSerializer(BaseReferenceModelSerializer):
    """Device Request Performer Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = DeviceRequestPerformerReference
        exclude = ["created_at", "updated_at"]


class DeviceRequestPerformerCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Device Request Performer Codeable Reference Serializer."""

    reference = DeviceRequestPerformerReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta."""

        model = DeviceRequestPerformerCodeableReference
        exclude = ["created_at", "updated_at"]


class DeviceRequestReasonReferenceSerializer(BaseReferenceModelSerializer):
    """Device Request Reason Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = DeviceRequestReasonReference
        exclude = ["created_at", "updated_at"]


class DeviceRequestReasonCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Device Request Reason Codeable Reference Serializer."""

    reference = DeviceRequestReasonReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta."""

        model = DeviceRequestReasonCodeableReference
        exclude = ["created_at", "updated_at"]


class DeviceRequestRequesterReferenceSerializer(BaseReferenceModelSerializer):
    """Device Request Requester Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = DeviceRequestRequesterReference
        exclude = ["created_at", "updated_at"]


class DeviceRequestRequesterCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Device Request Requester Codeable Reference Serializer."""

    reference = DeviceRequestRequesterReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta."""

        model = DeviceRequestRequesterCodeableReference
        exclude = ["created_at", "updated_at"]


class DeviceRequestSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """Device Request Subject Reference Serializer."""

    identifier = IdentifierSerializer(required=False, many=False)

    class Meta:
        """Meta."""

        model = DeviceRequestSubjectReference
        exclude = ["created_at", "updated_at"]


class DeviceRequestSerializer(BaseWritableNestedModelSerializer):
    """Device Request Serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    based_on = ReferenceSerializer(many=True, required=False)
    replaces = DeviceRequestReferenceSerializer(many=True, required=False)
    group_identifier = IdentifierSerializer(many=False, required=False)
    # code = DeviceDeviceDefinitionCodeableReferenceSerializer(many=False, required=False)
    parameter = DeviceRequestParameterSerializer(many=True, required=False)
    subject = DeviceRequestSubjectReferenceSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    occurrence_period = PeriodSerializer(many=False, required=False)
    occurrence_timing = TimingSerializer(many=False, required=False)
    requester = DeviceRequestRequesterReferenceSerializer(many=False, required=False)
    performer = DeviceRequestPerformerCodeableReferenceSerializer(
        many=False, required=False
    )
    reason = DeviceRequestReasonCodeableReferenceSerializer(many=True, required=False)
    as_needed_for = CodeableConceptSerializer(many=False, required=False)
    # insurance = CoverageClaimReferenceSerializer(many=True, required=False)
    supporting_info = ReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)
    relevant_history = ProvenanceReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta."""

        model = DeviceRequest
        exclude = ["created_at", "updated_at"]



"""activity definitions serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.activitydefinitions.models import (
    ActivityDefinition,
    ActivityDefinitionDynamicValue,
    ActivityDefinitionParticipant,
    ActivityDefinitionParticipantTypeReference,
    ActivityDefinitionPlanDefinitionReference,
    ActivityDefinitionProductProductReference,
    ActivityDefinitionReference,
    ActivityDefinitionSubjectReference,
)
from dfhir.base.serializers import (
    AgeSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodingSerializer,
    ContactDetailSerializer,
    DurationSerializer,
    ExpressionSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    QuantitySerializer,
    RangeSerializer,
    RelatedArtifactSerializer,
    TimingSerializer,
    UsageContextSerializer,
)
from dfhir.locations.serializers import LocationReferenceSerializer


class ActiviyDefinitionPlanDefinitionReferenceSerializer(BaseReferenceModelSerializer):
    """activity plan definition reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ActivityDefinitionPlanDefinitionReference
        exclude = ["created_at", "updated_at"]


class ActivityDefinitionProductProductReferenceSerializer(BaseReferenceModelSerializer):
    """activity definition product reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ActivityDefinitionProductProductReference
        exclude = ["created_at", "updated_at"]


class ActivityDefinitionSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """activity definition subject reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ActivityDefinitionSubjectReference
        exclude = ["created_at", "updated_at"]


class ActivityDefinitionReferenceSerializer(BaseReferenceModelSerializer):
    """activity definition reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ActivityDefinitionReference
        exclude = ["created_at", "updated_at"]


class ActivityDefinitionParticipantTypeReferenceSerializer(
    BaseReferenceModelSerializer
):
    """activity definition participant type reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = ActivityDefinitionParticipantTypeReference
        exclude = ["created_at", "updated_at"]


class ActivityDefinitionParticipantSerializer(WritableNestedModelSerializer):
    """activity definition participant serializer."""

    # type_canonical = CapabilityStatementCanonicalSerializer(many=False, required=False)
    type_reference = ActivityDefinitionParticipantTypeReferenceSerializer(
        many=False, required=False
    )
    role = CodeableConceptSerializer(required=False)
    function = CodeableConceptSerializer(required=False)

    class Meta:
        """meta options."""

        model = ActivityDefinitionParticipant
        exclude = ["created_at", "updated_at"]


class ActivityDefinitionDynamicValueSerializer(WritableNestedModelSerializer):
    """activity definition dynamic value serializer."""

    expression = ExpressionSerializer(required=False)

    class Meta:
        """meta options."""

        model = ActivityDefinitionDynamicValue
        exclude = ["created_at", "updated_at"]


class ActivityDefinitionSerializer(BaseWritableNestedModelSerializer):
    """activity definition serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    version_algorithm_coding = CodingSerializer(required=False)
    subject_codeable_concept = CodeableConceptSerializer(required=False)
    subject_reference = ActivityDefinitionSubjectReferenceSerializer(
        many=False, required=False
    )
    # subject_canonical = EvidenceVariableSerializer(required=False)
    contact = ContactDetailSerializer(required=False, many=True)
    use_context = UsageContextSerializer(many=True, required=False)
    jurisdiction = CodeableConceptSerializer(many=True, required=False)
    effective_period = PeriodSerializer(required=False)
    topic = CodeableConceptSerializer(many=True, required=False)
    author = ContactDetailSerializer(many=True, required=False)
    editor = ContactDetailSerializer(many=True, required=False)
    reviewer = ContactDetailSerializer(many=True, required=False)
    endorser = ContactDetailSerializer(many=True, required=False)
    kind = CodingSerializer(required=False)
    related_artifact = RelatedArtifactSerializer(many=True, required=False)
    # profile = StructuredDefinitionCanonical(required=False)
    # library = LibraryCanonicalReference(required=False)
    code = CodeableConceptSerializer(required=False)
    timing_timing = TimingSerializer(required=False)
    timing_age = AgeSerializer(required=False)
    timing_range = RangeSerializer(required=False)
    timing_duration = DurationSerializer(required=False)
    as_needed_codeable_concept = CodeableConceptSerializer(required=False)
    location = LocationReferenceSerializer(required=False)
    participant = ActivityDefinitionParticipantSerializer(many=True, required=False)
    product_reference = ActivityDefinitionProductProductReferenceSerializer(
        required=False
    )
    product_codeable_concept = CodeableConceptSerializer(required=False)
    quantity = QuantitySerializer(required=False)
    # dosage = DosageSerializer(required=False, many=True)
    body_site = CodeableConceptSerializer(many=True, required=False)
    # subject_requirements = SpecimenDefinitionCanonnicalSerializer(required=False)
    # observation_requirement = ObservationDefinitionCanonicalSerializer(required=False)
    # observation_result_requirement = ObservationDefinitionCanonicalSerializer(required=False)
    # transform = StructuredDefinitionCanonicalSerializer(required=False)
    dynamic_value = ActivityDefinitionDynamicValueSerializer(many=True, required=False)

    class Meta:
        """meta options."""

        model = ActivityDefinition
        exclude = ["created_at", "updated_at"]



"""Biologically Derived Products views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import BiologicallyDerivedProduct
from .serializers import BiologicallyDerivedProductSerializer


class BiologicallyDerivedProductListView(APIView):
    """Biologically Derived Product list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: BiologicallyDerivedProductSerializer(many=True)})
    def get(self, request):
        """Get biologically derived product list."""
        biologicallyderivedproducts = BiologicallyDerivedProduct.objects.all()
        serializer = BiologicallyDerivedProductSerializer(
            biologicallyderivedproducts, many=True
        )
        return Response(serializer.data)

    @extend_schema(
        request=BiologicallyDerivedProductSerializer,
        responses={201: BiologicallyDerivedProductSerializer},
    )
    def post(self, request):
        """Create a biologically derived product."""
        serializer = BiologicallyDerivedProductSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class BiologicallyDerivedProductDetailView(APIView):
    """Biologically Derived Product detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get biologically derived product object."""
        try:
            return BiologicallyDerivedProduct.objects.get(pk=pk)
        except BiologicallyDerivedProduct.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: BiologicallyDerivedProductSerializer})
    def get(self, request, pk=None):
        """Get biologically derived product detail."""
        queryset = self.get_object(pk)
        serializer = BiologicallyDerivedProductSerializer(queryset)
        return Response(serializer.data)

    @extend_schema(
        request=BiologicallyDerivedProductSerializer,
        responses={200: BiologicallyDerivedProductSerializer},
    )
    def patch(self, request, pk=None):
        """Update biologically derived product."""
        queryset = self.get_object(pk)
        serializer = BiologicallyDerivedProductSerializer(queryset, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a biologically derived product."""
        biologically_derived_product = self.get_object(pk)
        biologically_derived_product.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""medication statement urls."""

from django.urls import path

from dfhir.medicationstatements import views

app_name = "medicationstatements"


urlpatterns = [
    path(
        "medicationstatements/",
        views.MedicationStatementListView.as_view(),
        name="medicationstatement-list",
    ),
    path(
        "medicationstatements/<int:pk>/",
        views.MedicationStatementDetailedView.as_view(),
        name="medicationstatement-detail",
    ),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('accounts', '0003_initial'),
        ('base', '0004_initial'),
        ('claimresponses', '0003_initial'),
        ('encounters', '0003_initial'),
        ('locations', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='PaymentReconciliationAllocationTargetReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PaymentReconciliationPaymentIssuerReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PaymentReconciliationProcessNote',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.CharField(choices=[('display', 'Display'), ('print', 'Print'), ('printoper', 'Print Oper')], max_length=255, null=True)),
                ('text', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PaymentReconciliationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OrganizationPractitionerPractitionerRoleReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='organization_practitioner_practitioner_role_reference_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='organization_practitioner_practitioner_role_reference_organization', to='organizations.organization')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='organization_practitioner_practitioner_role_reference_patient', to='patients.patientreference')),
                ('related_person', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='organization_practitioner_practitioner_role_reference_related_person', to='relatedpersons.relatedperson')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PaymentReconciliation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('cancelled', 'Cancelled'), ('draft', 'Draft'), ('entered-in-error', 'Entered in Error')], max_length=255, null=True)),
                ('created', models.DateTimeField(null=True)),
                ('outcome', models.CharField(choices=[('queued', 'Queued'), ('complete', 'Complete'), ('error', 'Error'), ('partial', 'Partial')], max_length=255, null=True)),
                ('disposition', models.CharField(max_length=255, null=True)),
                ('date', models.DateField(null=True)),
                ('card_brand', models.CharField(max_length=255, null=True)),
                ('account_number', models.CharField(max_length=255, null=True)),
                ('expiration_date', models.DateField(null=True)),
                ('processor', models.CharField(max_length=255, null=True)),
                ('reference_number', models.CharField(max_length=255, null=True)),
                ('authorization', models.DateField(null=True)),
                ('amount', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_amount', to='base.money')),
                ('enterer', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_enterer', to='paymentreconciliations.organizationpractitionerpractitionerrolereference')),
                ('form_code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_form_code', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='payment_reconciliation_identifier', to='base.identifier')),
                ('issuer_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_issuer_type', to='base.codeableconcept')),
                ('kind', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_kind', to='base.codeableconcept')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_location', to='locations.locationreference')),
                ('method', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_method', to='base.codeableconcept')),
                ('payment_identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_payment_identifier', to='base.identifier')),
                ('period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_period', to='base.period')),
                ('requestor', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_requestor', to='paymentreconciliations.organizationpractitionerpractitionerrolereference')),
                ('returned_amount', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_returned_amount', to='base.money')),
                ('tendered_amount', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_tendered_amount', to='base.money')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PaymentReconciliationAllocation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('target_item_string', models.CharField(max_length=255, null=True)),
                ('target_item_positive_int', models.PositiveIntegerField(null=True)),
                ('date', models.DateField(null=True)),
                ('account', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_account', to='accounts.accountreference')),
                ('amount', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_amount', to='base.money')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_encounter', to='encounters.encounterreference')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_identifier', to='base.identifier')),
                ('payee', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_payee', to='paymentreconciliations.organizationpractitionerpractitionerrolereference')),
                ('predecessor', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_predecessor', to='base.identifier')),
                ('response', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='payment_reconciliation_allocation_response', to='claimresponses.claimresponse')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""supply request serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    QuantitySerializer,
    RangeSerializer,
    ReferenceSerializer,
)
from dfhir.patients.serializers import PatientReferenceSerializer
from dfhir.supplyrequests.models import (
    SupplyRequest,
    SupplyRequestDeliverFromReference,
    SupplyRequestDeliverToReference,
    SupplyRequestItemCodeableReference,
    SupplyRequestItemReference,
    SupplyRequestParameter,
    SupplyRequestReasonCodeableReference,
    SupplyRequestReasonReference,
    SupplyRequestReference,
    SupplyRequestRequesterReference,
    SupplyRequestSupplierReference,
)


class SupplyRequestParameterSerializer(WritableNestedModelSerializer):
    """supply request parameter serializer."""

    code = CodeableConceptSerializer(required=False)
    value_codeable_concept = CodeableConceptSerializer(required=False)
    value_quantity = QuantitySerializer(required=False)
    value_range = RangeSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequestParameter
        exclude = ["created_at", "updated_at"]


class SupplyRequestRequesterReferenceSerializer(BaseReferenceModelSerializer):
    """supply request requester reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequestRequesterReference
        exclude = ["created_at", "updated_at"]


class SupplyRequestReasonReferenceSerializer(BaseReferenceModelSerializer):
    """supply request reason reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequestReasonReference
        exclude = ["created_at", "updated_at"]


class SupplyRequestReasonCodeableReferenceSerializer(WritableNestedModelSerializer):
    """supply request reason codeable reference serializer."""

    concept = CodeableConceptSerializer(required=False)
    reference = SupplyRequestReasonReferenceSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequestReasonCodeableReference
        exclude = ["created_at", "updated_at"]


class SupplyRequestItemReferenceSerializer(BaseReferenceModelSerializer):
    """supply request item reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequestItemReference
        exclude = ["created_at", "updated_at"]


class SupplyRequestSupplierReferenceSerializer(BaseReferenceModelSerializer):
    """supply request supplier reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequestSupplierReference
        exclude = ["created_at", "updated_at"]


class SupplyRequestDeliverToReferenceSerializer(BaseReferenceModelSerializer):
    """supply request deliver to reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequestDeliverToReference
        exclude = ["created_at", "updated_at"]


class SupplyRequestDeliverFromReferenceSerializer(BaseReferenceModelSerializer):
    """supply request deliver from reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequestDeliverFromReference
        exclude = ["created_at", "updated_at"]


class SupplyRequestItemCodeableReferenceSerializer(WritableNestedModelSerializer):
    """supply request item codeable reference serializer."""

    concept = CodeableConceptSerializer(required=False)
    reference = SupplyRequestItemReferenceSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequestItemCodeableReference
        exclude = ["created_at", "updated_at"]


class SupplyRequestSerializer(BaseWritableNestedModelSerializer):
    """supply request serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    based_on = ReferenceSerializer(required=False, many=True)
    category = CodeableConceptSerializer(required=False)
    deliver_for = PatientReferenceSerializer(required=False)
    item = SupplyRequestItemCodeableReferenceSerializer(required=False)
    quantity = QuantitySerializer(required=False)
    parameter = SupplyRequestParameterSerializer(required=False, many=True)
    occurrence_period = PeriodSerializer(required=False)
    occurrence_timing = PeriodSerializer(required=False)
    requester = SupplyRequestRequesterReferenceSerializer(required=False)
    supplier = SupplyRequestSupplierReferenceSerializer(required=False, many=True)
    reason = SupplyRequestReasonCodeableReferenceSerializer(required=False, many=True)
    deliver_from = SupplyRequestDeliverFromReferenceSerializer(required=False)
    deliver_to = SupplyRequestDeliverToReferenceSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequest
        exclude = ["created_at", "updated_at"]


class SupplyRequestReferenceSerilizer(BaseReferenceModelSerializer):
    """supply request reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta options."""

        model = SupplyRequestReference
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='MedicationBatch',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('lot_number', models.CharField(max_length=255, null=True)),
                ('expiration_date', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationCodes',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationDoseForm',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationIngredientItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationIngredient',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('is_active', models.BooleanField(default=True)),
                ('strength_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='mediation_ingredient_strength_codeable_concept', to='base.codeableconcept')),
                ('strength_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='mediation_ingredient_strength_quantity', to='base.quantity')),
                ('strength_ratio', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='mediation_ingredient_strength_ratio', to='base.ratio')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Medication',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('inactive', 'Inactive'), ('entered_in_error', 'Entered in error')], max_length=255, null=True)),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_code', to='base.codeableconcept')),
                ('dose_form', models.OneToOneField(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_dose_form', to='base.codeableconcept')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_identifier', to='base.identifier')),
                ('marketing_authorization_holder', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_organization', to='base.organizationreference')),
                ('total_volume', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_total_volume', to='base.quantity')),
                ('batch', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_batch', to='medications.medicationbatch')),
                ('ingredient', models.ManyToManyField(blank=True, related_name='medication_ingredient', to='medications.medicationingredient')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_reference_identifier', to='base.identifier')),
                ('medication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_reference_medication', to='medications.medication')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_codeable_reference_concept', to='base.codeableconcept')),
                ('reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_codeable_reference_reference', to='medications.medicationreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MedicationSubstanceCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_substance_codeable_reference_concept', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='medicationingredient',
            name='item',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='mediation_ingredient_item', to='medications.medicationsubstancecodeablereference'),
        ),
        migrations.CreateModel(
            name='MedicationSubstanceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_substance_reference_identifier', to='base.identifier')),
                ('medication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_substance_reference', to='medications.medication')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='medicationsubstancecodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_substance_codeable_reference_reference', to='medications.medicationsubstancereference'),
        ),
    ]



"""Appointments URL Configuration."""

from django.urls import path

from .views import AppointmentDetailView, AppointmentListView

app_name = "appointments"

urlpatterns = [
    path("appointments/", AppointmentListView.as_view(), name="appointment-list"),
    path(
        "appointments/<int:pk>/",
        AppointmentDetailView.as_view(),
        name="appointment-detail",
    ),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('medicationrequests', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitioners', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='medicationrequest',
            name='recorder',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_recorder', to='practitioners.practitionerpractitionerrolereference'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='status_reason',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_status_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='medicationrequest',
            name='supporting_information',
            field=models.ManyToManyField(blank=True, related_name='medication_request_supporting_information', to='base.reference'),
        ),
        migrations.AddField(
            model_name='medicationrequestbasedonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_based_on_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationrequestbasedonreference',
            name='medication_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_request_based_on_reference', to='medicationrequests.medicationrequest'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('genomicstudies', '0001_initial'),
        ('observations', '0002_initial'),
        ('practitioners', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='genomicstudy',
            name='interpreter',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_interpreter', to='practitioners.practitionerpractitionerrolereference'),
        ),
        migrations.AddField(
            model_name='genomicstudy',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='genomicstudy',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_reason', to='conditions.conditionobservationcodeablereference'),
        ),
        migrations.AddField(
            model_name='genomicstudy',
            name='referrer',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_referrer', to='practitioners.practitionerpractitionerrolereference'),
        ),
        migrations.AddField(
            model_name='genomicstudy',
            name='type',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='change_type',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_change_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='focus',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_focus', to='base.reference'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='genomic_build',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_genomic_build', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='method_type',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_method_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='regions_called',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_regions_called', to='observations.documentreferenceobservationreference'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='regions_studied',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_regions_studied', to='observations.documentreferenceobservationreference'),
        ),
    ]



"""Device definition app config."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DevicedefinitionsConfig(AppConfig):
    """Device Definition config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.devicedefinitions"
    verbose_name = _("Device Definitions")



"""specimen urls."""

from django.urls import path

from dfhir.specimens import views

app_name = "specimens"


urlpatterns = [
    path("specimens/", views.SpecimenListView.as_view()),
    path("specimens/<int:pk>/", views.SpecimenDetailView.as_view()),
]



"""formulary item views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.formularyitems.models import FormularyItem
from dfhir.formularyitems.serializers import FormularyItemSerializer


class FormularyItemListView(APIView):
    """formulary item list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: FormularyItemSerializer(many=True)})
    def get(self, request):
        """Get request."""
        formulary_items = FormularyItem.objects.all()
        serializer = FormularyItemSerializer(formulary_items, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=FormularyItemSerializer, responses={200: FormularyItemSerializer}
    )
    def post(self, request):
        """Post request."""
        serializer = FormularyItemSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class FormularyItemDetailView(APIView):
    """formulary item detail view."""

    def get_object(self, pk):
        """Get object."""
        try:
            return FormularyItem.objects.get(pk=pk)
        except FormularyItem.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: FormularyItemSerializer})
    def get(self, request, pk):
        """Get request."""
        formulary_item = self.get_object(pk)
        serializer = FormularyItemSerializer(formulary_item)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=FormularyItemSerializer, responses={200: FormularyItemSerializer}
    )
    def patch(self, request, pk):
        """Patch request."""
        formulary_item = self.get_object(pk)
        serializer = FormularyItemSerializer(
            formulary_item, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete request."""
        formulary_item = self.get_object(pk)
        formulary_item.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('devices', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('schedules', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='signatureonbehalfofreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_on_behalf_of_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='signatureonbehalfofreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_on_behalf_of_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='signatureonbehalfofreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_on_behalf_of_reference_related_person', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='signature',
            name='on_behalf_of',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_on_behalf_of', to='base.signatureonbehalfofreference'),
        ),
        migrations.AddField(
            model_name='signaturewhoreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_who_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='signaturewhoreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_who_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='signaturewhoreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_who_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='signaturewhoreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_who_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='signaturewhoreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_who_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='signaturewhoreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_who_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='signaturewhoreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_who_reference_related_person', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='signature',
            name='who',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='signature_who', to='base.signaturewhoreference'),
        ),
        migrations.AddField(
            model_name='timing',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='timing_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='timing',
            name='repeat',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='timing_repeat', to='base.repeat'),
        ),
        migrations.AddField(
            model_name='triggerdefinition',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='trigger_definition_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='triggerdefinition',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='trigger_definition_condition', to='base.expression'),
        ),
        migrations.AddField(
            model_name='triggerdefinition',
            name='timing_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='trigger_definition_timing_reference', to='schedules.schedulereference'),
        ),
        migrations.AddField(
            model_name='triggerdefinition',
            name='timing_timing',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='trigger_definition_timing_timing', to='base.timing'),
        ),
        migrations.AddField(
            model_name='usagecontext',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='usage_context_code', to='base.coding'),
        ),
        migrations.AddField(
            model_name='usagecontext',
            name='value_codeable_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='usage_context_value', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='usagecontext',
            name='value_quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='usage_context_value_quantity', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='usagecontext',
            name='value_range',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='usage_context_value_range', to='base.range'),
        ),
        migrations.AddField(
            model_name='virtualservicedetailaddress',
            name='address_contact_point',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='virtual_service_detail_address_contact_point', to='base.contactpoint'),
        ),
        migrations.AddField(
            model_name='virtualservicedetailaddress',
            name='address_extended_contact_detail',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='virtual_service_detail_address_extended_contact_detail', to='base.extendedcontactdetail'),
        ),
        migrations.AddField(
            model_name='virtualservicedetails',
            name='address',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='virtual_service_details_address', to='base.virtualservicedetailaddress'),
        ),
        migrations.AddField(
            model_name='virtualservicedetails',
            name='channel_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='virtual_service_details_channel_type', to='base.coding'),
        ),
    ]



"""condition urls."""

from django.urls import path

from dfhir.conditions.views import ConditionDetailView, ConditionListView

app_name = "conditions"

urlpatterns = [
    path("conditions/", ConditionListView.as_view(), name="condition-list"),
    path(
        "conditions/<int:pk>/", ConditionDetailView.as_view(), name="condition-detail"
    ),
]



"""Choices for the persons app."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class LinkAssuranceChoices(models.TextChoices):
    """Link Assurance Choices."""

    LEVEL1 = "level1", _("level1")
    LEVEL2 = "level2", _("level2")
    LEVEL3 = "level3", _("level3")
    LEVEL4 = "level4", _("level4")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('medicationrequests', '0001_initial'),
        ('molecularsequences', '0001_initial'),
        ('observations', '0001_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='observation',
            name='specimen',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_specimen', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='observation',
            name='value_attachment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_value_attachment', to='base.attachment'),
        ),
        migrations.AddField(
            model_name='observation',
            name='value_codeable_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_value_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='observation',
            name='value_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_value_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='observation',
            name='value_quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='observation_value', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='observation',
            name='value_range',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_value_range', to='base.range'),
        ),
        migrations.AddField(
            model_name='observation',
            name='value_ratio',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_value_ratio', to='base.ratio'),
        ),
        migrations.AddField(
            model_name='observation',
            name='value_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='molecularsequences.molecularsequencereference'),
        ),
        migrations.AddField(
            model_name='documentreferenceobservationreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='document_reference_observation_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='observationbasedonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_based_on_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='observationbasedonreference',
            name='medication_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='observation_based_on_reference_medication_request', to='medicationrequests.medicationrequest'),
        ),
    ]



"""document reference serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodeableReferenceSerializer,
    CodingSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    ReferenceSerializer,
)
from dfhir.bodystructures.serializers import (
    BodyStructureCodeableReferenceSerializer,
    BodyStructureReferenceSerializer,
)
from dfhir.documentreferences.models import (
    DocumentReference,
    DocumentReferenceAttester,
    DocumentReferenceAttesterPartyReference,
    DocumentReferenceAuthorReference,
    DocumentReferenceBasedOnReference,
    DocumentReferenceContent,
    DocumentReferenceContentProfile,
    DocumentReferenceContextReference,
    DocumentReferenceReference,
    DocumentReferenceRelatesTo,
)


class DocumentReferenceContextReferenceSerializer(BaseReferenceModelSerializer):
    """Serializer for DocumentReferenceContextReference."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = DocumentReferenceContextReference
        exclude = ["created_at", "updated_at"]


class DocumentReferenceReferenceSerializer(BaseReferenceModelSerializer):
    """Serializer for DocumentReferenceReference."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = DocumentReferenceReference
        exclude = ["created_at", "updated_at"]


class DocumentReferenceBasedOnReferenceSerializer(BaseReferenceModelSerializer):
    """Serializer for DocumentReferenceBasedOnReference."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = DocumentReferenceBasedOnReference
        exclude = ["created_at", "updated_at"]


class DocumentReferenceAuthorReferenceSerializer(BaseReferenceModelSerializer):
    """Serializer for DocumentReferenceAuthorReference."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = DocumentReferenceAuthorReference
        exclude = ["created_at", "updated_at"]


class DocumentReferenceAttesterPartyReferenceSerializer(BaseReferenceModelSerializer):
    """Serializer for DocumentReferenceAttesterPartyReference."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = DocumentReferenceAttesterPartyReference
        exclude = ["created_at", "updated_at"]


class DocumentReferenceAttesterSerializer(WritableNestedModelSerializer):
    """Serializer for DocumentReferenceAttester."""

    mode = CodeableConceptSerializer(required=False)
    party = DocumentReferenceAttesterPartyReferenceSerializer(required=False)

    class Meta:
        """meta options."""

        model = DocumentReferenceAttester
        exclude = ["created_at", "updated_at"]


class DocumentReferenceRelatesToSerializer(WritableNestedModelSerializer):
    """Serializer for DocumentReferenceRelatesTo."""

    code = CodeableConceptSerializer(required=False)
    target = DocumentReferenceReferenceSerializer(required=False)

    class Meta:
        """meta options."""

        model = DocumentReferenceRelatesTo
        exclude = ["created_at", "updated_at"]


class DocumentReferenceContentProfileSerializer(WritableNestedModelSerializer):
    """Serializer for DocumentReferenceContentProfile."""

    value_coding = CodingSerializer(required=False)

    class Meta:
        """meta options."""

        model = DocumentReferenceContentProfile
        exclude = ["created_at", "updated_at"]


class DocumentReferenceContentSerializer(WritableNestedModelSerializer):
    """Serializer for DocumentReferenceContent."""

    attachment = AttachmentSerializer(required=False)
    profile = DocumentReferenceContentProfileSerializer(required=False, many=True)

    class Meta:
        """meta options."""

        model = DocumentReferenceContent
        exclude = ["created_at", "updated_at"]


class DocumentReferenceSerializer(BaseWritableNestedModelSerializer):
    """document reference writable serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    based_on = DocumentReferenceBasedOnReferenceSerializer(required=False, many=True)
    modality = CodeableConceptSerializer(required=False, many=True)
    type = CodeableConceptSerializer(required=False)
    category = CodeableConceptSerializer(required=False, many=True)
    subject = ReferenceSerializer(required=False)
    context = DocumentReferenceContextReferenceSerializer(required=False, many=True)
    event = CodeableReferenceSerializer(required=False, many=True)
    related = ReferenceSerializer(required=False, many=True)
    body_site = BodyStructureReferenceSerializer(required=False, many=True)
    body_site = BodyStructureCodeableReferenceSerializer(required=False, many=True)
    facility_type = CodeableConceptSerializer(required=False)
    practice_setting = CodeableConceptSerializer(required=False)
    period = PeriodSerializer(required=False)
    author = DocumentReferenceAuthorReferenceSerializer(required=False, many=True)
    attester = DocumentReferenceAttesterSerializer(required=False, many=True)
    custodian = OrganizationReferenceSerializer(required=False)
    relates_to = DocumentReferenceRelatesToSerializer(required=False, many=True)
    security_label = CodeableConceptSerializer(required=False, many=True)
    content = DocumentReferenceContentSerializer(required=False, many=True)

    class Meta:
        """meta options."""

        model = DocumentReference
        exclude = ["created_at", "updated_at"]



"""CoverageEligibilityRequests models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Money,
    OrganizationReference,
    Period,
    SimpleQuantity,
    TimeStampedModel,
)

from . import choices


class CoverageEligibilityRequestEvent(TimeStampedModel):
    """CoverageEligibilityRequestEvent model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_event_type",
        null=True,
    )
    when_date_time = models.DateTimeField(null=True)
    when_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_event_when_period",
        null=True,
    )


class CoverageEligibilityRequestSupportingInfoInformationReference(BaseReference):
    """CoverageEligibilityRequestSupportingInfoInformationReference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_supporting_info_information_reference_identifier",
        null=True,
    )


class CoverageEligibilityRequestSupportingInfo(TimeStampedModel):
    """CoverageEligibilityRequestSupportingInfo model."""

    sequence = models.IntegerField()
    information = models.ForeignKey(
        CoverageEligibilityRequestSupportingInfoInformationReference,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_supporting_info_information",
        null=True,
    )
    applies_to_all = models.BooleanField(default=False)


class CoverageEligibilityRequestInsurance(TimeStampedModel):
    """CoverageEligibilityRequestInsurance model."""

    focal = models.BooleanField(default=False)
    coverage = models.ForeignKey(
        "coverages.CoverageReference",
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_insurance_coverage",
        null=True,
    )
    business_arrangement = models.CharField(max_length=255, null=True)


class CoverageEligibilityRequestItemDiagnosis(TimeStampedModel):
    """CoverageEligibilityRequestItemDiagnosis model."""

    diagnosis_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_item_diagnosis_codeable_concept",
        null=True,
    )
    diagnosis_reference = models.ForeignKey(
        "conditions.ConditionReference",
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_item_diagnosis_reference",
        null=True,
    )


class CoverageEligibilityRequestItemDetailReference(BaseReference):
    """CoverageEligibilityRequestItemDetailReference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_item_detail_reference_identifier",
        null=True,
    )


class CoverageEligibilityRequestItem(TimeStampedModel):
    """CoverageEligibilityRequestItem model."""

    supporting_info_sequence = ArrayField(models.IntegerField(), null=True)
    category = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_item_category",
        null=True,
    )
    product_or_service = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_item_product_or_service",
        null=True,
    )
    modifier = models.ManyToManyField(
        CodeableConcept,
        related_name="coverage_eligibility_request_item_modifier",
        blank=True,
    )
    provider = models.ForeignKey(
        "practitioners.PractitionerPractitionerRoleReference",
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_item_provider",
        null=True,
    )
    quantity = models.ForeignKey(
        SimpleQuantity,
        on_delete=models.CASCADE,
        null=True,
        related_name="coverage_eligibility_request_item_quantity",
    )
    unit_price = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        null=True,
        related_name="coverage_eligibility_request_item_unit_price",
    )
    facility = models.ForeignKey(
        "locations.LocationOrganizationReference",
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_item_facility",
        null=True,
    )
    diagnosis = models.ManyToManyField(
        CoverageEligibilityRequestItemDiagnosis,
        related_name="coverage_eligibility_request_item_diagnosis",
        blank=True,
    )
    detail = models.ManyToManyField(
        CoverageEligibilityRequestItemDetailReference,
        related_name="coverage_eligibility_request_item_detail",
        blank=True,
    )


class CoverageEligibilityRequestProviderReference(BaseReference):
    """CoverageEligibilityRequestProviderReference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_provider_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_provider_reference_practitioner",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_provider_reference_practitioner_role",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_provider_reference_organization",
        null=True,
    )


class CoverageEligibilityRequest(TimeStampedModel):
    """CoverageEligibilityRequest model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="coverage_eligibility_request_identifier", blank=True
    )
    status = models.CharField(
        max_length=255, choices=choices.CoverageEligibilityRequestStatus.choices
    )
    priority = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_priority",
        null=True,
    )
    purpose = ArrayField(
        models.CharField(
            max_length=255, choices=choices.CoverageEligibilityRequestPurpose.choices
        ),
        null=True,
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_patient",
        null=True,
    )
    event = models.ManyToManyField(
        CoverageEligibilityRequestEvent,
        related_name="coverage_eligibility_request_event",
        blank=True,
    )
    service_date = models.DateField(null=True)
    service_period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_service_period",
        null=True,
    )
    created = models.DateTimeField(null=True)
    enterer = models.ForeignKey(
        "practitioners.PractitionerPractitionerRoleReference",
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_enterer",
        null=True,
    )
    provider = models.ForeignKey(
        CoverageEligibilityRequestProviderReference,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_provider",
        null=True,
    )
    insurer = models.ForeignKey(
        OrganizationReference,
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_insurer",
        null=True,
    )
    facility = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.CASCADE,
        related_name="coverage_eligibility_request_facility",
        null=True,
    )
    supporting_info = models.ManyToManyField(
        CoverageEligibilityRequestSupportingInfo,
        related_name="coverage_eligibility_request_supporting_info",
        blank=True,
    )
    insurance = models.ManyToManyField(
        CoverageEligibilityRequestInsurance,
        related_name="coverage_eligibility_request_insurance",
        blank=True,
    )
    item = models.ManyToManyField(
        CoverageEligibilityRequestItem,
        related_name="coverage_eligibility_request_item",
        blank=True,
    )



"""Appointments models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Coding,
    Identifier,
    Period,
    Reference,
    TimeStampedModel,
    VirtualServiceDetails,
)
from dfhir.healthcareservices.models import HealthCareServiceCodeableReference
from dfhir.locations.models import Location
from dfhir.practitioners.models import Practitioner

from .choices import (
    AppointmentStatus,
    ParticipationStatusChoices,
)


class AppointmentReasonReference(BaseReference):
    """Reference to the condition, procedure, observation, or immunization recommendation."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_reason_identifier",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_reason_observation",
    )


class AppointmentReasonCodeableReference(TimeStampedModel):
    """Reference to the condition, procedure, observation, or immunization recommendation."""

    reference = models.ForeignKey(
        AppointmentReasonReference,
        on_delete=models.DO_NOTHING,
        related_name="appointment_reason_reference",
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="appointment_reason_concept",
        null=True,
    )


class WeeklyTemplate(TimeStampedModel):
    """WeeklyTemplate model is used to store the weekly recurrence pattern."""

    monday = models.BooleanField(default=False)
    tuesday = models.BooleanField(default=False)
    wednesday = models.BooleanField(default=False)
    thursday = models.BooleanField(default=False)
    friday = models.BooleanField(default=False)
    saturday = models.BooleanField(default=False)
    sunday = models.BooleanField(default=False)
    week_interval = models.PositiveIntegerField(null=True)


class MonthlyTemplate(TimeStampedModel):
    """MonthlyTemplate model is used to store the monthly recurrence pattern."""

    day_of_month = models.PositiveIntegerField()
    day_of_week = models.ForeignKey(
        Coding,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="monthly_template_day_of_week",
    )
    nth_week_of_month = models.ForeignKey(
        Coding,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="monthly_template_nth_week_of_month",
    )
    month_interval = models.PositiveIntegerField()


class YearlyTemplate(TimeStampedModel):
    """YearlyTemplate model is used to store the yearly recurrence pattern."""

    year_interval = models.PositiveIntegerField()


class RecurrenceTemplate(TimeStampedModel):
    """Details of the recurrence pattern/template used to generate occurrences."""

    recurrence_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="recurrence_type",
    )
    last_occurrence_date = models.DateField(null=True, blank=True)
    occurrence_count = models.PositiveIntegerField(null=True, blank=True)
    occurrence_date = models.DateField(null=True, blank=True)
    excluding_recurrence_id = ArrayField(models.PositiveIntegerField(), null=True)
    time_zone = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="recurrence_time_zone",
    )
    weekly_template = models.ForeignKey(
        WeeklyTemplate, on_delete=models.DO_NOTHING, null=True
    )
    monthly_template = models.ForeignKey(
        MonthlyTemplate, on_delete=models.DO_NOTHING, null=True
    )
    yearly_template = models.ForeignKey(
        YearlyTemplate, on_delete=models.DO_NOTHING, null=True
    )
    excluding_date = ArrayField(models.DateField(), null=True)


class AppointmentEncounterReason(TimeStampedModel):
    """Reason this appointment is scheduled."""

    display = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)


class AppointmentBasedOnReference(BaseReference):
    """Reference to the care plan, device request, medication request, service request, request orchestration, nutrition order, visual prescription, and immunization recommendation."""

    # TODO: Implement the commentetd models
    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_based_on_reference_identifier",
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_based_on_reference_care_plan",
    )
    device_request = models.ForeignKey(
        "devicerequests.DeviceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_based_on_reference_device_request",
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_based_on_reference_medication_request",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_based_on_reference_service_request",
    )
    # request_orchestration = models.ForeignKey(
    #     "requestorchestrations.RequestOrchestration",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="appointment_based_on_reference_request_orchestration",
    # )
    nutrition_order = models.ForeignKey(
        "nutritionorders.NutritionOrder",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_based_on_reference_nutrition_order",
    )
    # visual_prescription = models.ForeignKey(
    #     "visualprescriptions.VisualPrescription",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="appointment_based_on_reference_visual_prescription",
    # )


class DocumentReferenceBinaryCommunicationReference(BaseReference):
    """Reference to the patient instructions."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="document_reference_binary_communication_reference_identifier",
        null=True,
    )
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        related_name="document_reference_binary_communication_reference_document_reference",
        null=True,
    )
    # binary = models.ForeignKey(
    #     "binaries.Binary",
    #     on_delete=models.DO_NOTHING,
    #     related_name="document_reference_binary_communication_reference_binary",
    #     null=True,
    # )
    communication = models.ForeignKey(
        "communications.Communication",
        on_delete=models.DO_NOTHING,
        related_name="document_reference_binary_communication_reference_communication",
        null=True,
    )


class AppointmentParticipantActor(BaseReference):
    """Reference to the patient, group, practitioner, practitioner role, care team, related person, device, healthcare service, and location."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="appointment_participant_actor_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        related_name="appointment_participant_actor_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        Practitioner,
        on_delete=models.DO_NOTHING,
        related_name="appointment_participant_actor_practitioner",
        null=True,
    )
    healthcareservice = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        related_name="appointment_participant_actor_healthcareservice",
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        related_name="appointment_participant_actor_device",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        related_name="appointment_participant_actor_group",
        null=True,
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        related_name="appointment_participant_actor_care_team",
        null=True,
    )
    location = models.ForeignKey(
        Location,
        on_delete=models.DO_NOTHING,
        related_name="appointment_participant_actor_location",
        null=True,
    )


class AppointmentParticipant(TimeStampedModel):
    """Participants involved in the appointment."""

    actor = models.ForeignKey(
        AppointmentParticipantActor,
        on_delete=models.CASCADE,
        null=True,
        related_name="appointment_participant_actor",
    )
    type = models.ManyToManyField(
        CodeableConcept, related_name="appointment_participant_type", blank=True
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_participant_period",
    )
    required = models.BooleanField(default=False)
    status = models.CharField(
        max_length=255, choices=ParticipationStatusChoices.choices, null=True
    )


class AppointmentReference(BaseReference):
    """Appointment reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="appointment_reference_identifier",
        null=True,
    )
    appointment = models.ForeignKey(
        "appointments.Appointment",
        on_delete=models.CASCADE,
        related_name="appointment_reference_appointment",
        null=True,
    )


class Appointment(TimeStampedModel):
    """Appointment model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="appointment_identifier", blank=True
    )

    status = models.CharField(
        max_length=255, choices=AppointmentStatus.choices, null=True
    )
    cancellation_reason = models.ForeignKey(
        CodeableConcept, on_delete=models.DO_NOTHING, null=True
    )
    klass = models.ManyToManyField(
        CodeableConcept, related_name="appointment_class", blank=True
    )
    service_category = models.ManyToManyField(
        CodeableConcept, related_name="service_category", blank=True
    )
    service_type = models.ManyToManyField(
        HealthCareServiceCodeableReference,
        related_name="appointment_service_type",
        blank=True,
    )
    specialty = models.ManyToManyField(
        CodeableConcept, related_name="appointment_specialty", blank=True
    )
    appointment_type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_type",
    )
    reason = models.ManyToManyField(
        AppointmentReasonCodeableReference,
        related_name="appointment_reason",
        blank=True,
    )
    priority = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_priority",
    )
    description = models.TextField(blank=True, null=True)
    replaces = models.ManyToManyField(
        AppointmentReference, related_name="appointment_replaces", blank=True
    )
    virtual_service = models.ManyToManyField(
        VirtualServiceDetails,
        related_name="appointment_virtual_service_details",
        blank=True,
    )
    supporting_information = models.ManyToManyField(
        Reference, related_name="appointment_supporting_information", blank=True
    )
    previous_appointment = models.ForeignKey(
        AppointmentReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_previous_appointment",
    )
    originating_appointment = models.ForeignKey(
        AppointmentReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="appointment_originating_appointment",
    )
    start = models.DateTimeField(null=True)
    end = models.DateTimeField(null=True)
    minutes_duration = models.PositiveIntegerField(null=True)
    requested_period = models.ManyToManyField(
        Period, related_name="appointment_requested_period", blank=True
    )
    slot = models.ManyToManyField(
        "slots.SlotReference", related_name="appointment_slot", blank=True
    )
    # TODO: uncomment after implementing accounts model

    account = models.ManyToManyField(
        "accounts.AccountReference", related_name="appointment_account", blank=True
    )
    cancellation_date = models.DateTimeField(
        null=True, blank=True, help_text="Date of cancellation"
    )
    note = models.ManyToManyField(
        Annotation, related_name="appointment_note", blank=True
    )
    patient_instruction = models.ManyToManyField(
        DocumentReferenceBinaryCommunicationReference,
        related_name="appointment_patient_instruction",
        blank=True,
    )
    based_on = models.ManyToManyField(
        AppointmentBasedOnReference,
        related_name="appointment_based_on",
        blank=True,
    )
    recurrence_id = models.PositiveIntegerField(null=True, blank=True)
    occurrence_changed = models.BooleanField(default=False)
    subject = models.ForeignKey(
        "patients.PatientGroupReference",
        on_delete=models.DO_NOTHING,
        related_name="appointment_subject",
        null=True,
    )
    participant = models.ManyToManyField(
        AppointmentParticipant, related_name="appointment_participant", blank=True
    )
